//мк ATtiny2313
//F_CPU = 1000000

#include <avr/io.h>
#include <avr/interrupt.h>

#define NEW_MODE (mode & 0b00111111) == 0 		// Проверка на начало нового режима

typedef unsigned char uchar;		// обьявление что unsigned char это то же самое что и uchar


uchar PORTZ;	// здесь хранится состояние виртуального порта,
				// значение этой переменной должно меняться ТОЛЬКО с помощью функции set_virt_port()


int mode = 0;
/*	описание битов mode:
	0-3 bit - этап (4 бита, 16 тактов одного рисунка)
	4 bit - направление (влево/вправо)
	5 bit - счет/сдвиг
	6 bit -	движение по строке/диагонали
	7 bit - столбцу/диагонали
	8-10 bit - стартовая позиция
*/

// Массив стартовых позиций для режима сдвига
uchar start_pos[] = {0b00000001,
					 0b00001111,
					 0b00010001,
					 0b00011111,
					 0b01111111};



void set_virt_port(uchar port);		// Запись байта в "виртуальный" порт
void next_step(void);				// Перейти к следующему "шагу" в светодинамическом рисунке.
uchar make_move(uchar port);		// Определение режима: сдвиг или счет.

uchar shift(uchar port);			// Определение направления сдвига
uchar shift_left(uchar port);		// Сдвиг влево
uchar shift_right(uchar port);		// Сдвиг вправо

uchar count(uchar port);			// Определение направления счета
uchar count_left(uchar port);		// Счет влево
uchar count_right(uchar port);		// Счет вправо


ISR(TIMER1_COMPA_vect) {
    next_step();        // Перейти к следующему "шагу" в светодинамическом рисунке.
}



//========================> функция main() <==========================
int main(void)
{
	DDRA = 0b00000011;
	PORTA = 0b11111100;

	DDRD = 0b01111110;
	PORTD = 0b10000001;

	DDRB = 0b11111111;

	PORTB = 0xFF;				// строки, аноды

	set_virt_port(0xFF);		// столбцы, катоды

// Тактирование от таймера
// Частота тактирования (clock) вычисляется по следующей формуле:
// clock = F_CPU / tim_pre / (OCR1A + 1), где tim_pre - предделитель таймера.
// В нашем случае F_CPU = 1000000, tim_pre = 256, и OCR1A = 1024:
// clock = 1000000 / 256 / (1024+1) = 3.8 Гц
 
	TCCR1A = 0b00000000;
	TCCR1B = 0b00001100;		// Режим сброс по совпадению, предделитель на 256
	TCCR1C = 0b00000000;
	OCR1A = 1024;
	TIMSK = 0b01000000;			// Прерывание по совпадению

	sei();
	while (1) {}
}


//Запись байта в "виртуальный" порт
void set_virt_port(uchar port) {
	PORTZ = port;		//В PORTZ хранится актуальное состояние "виртуального" порта

	port^= 0xFF;

	PORTA |= 0b00000011;
	PORTD |= 0b01111110;

	PORTD &= (port << 1) | 0b11111101;		// PZ0 == PD1
	PORTA &=  port | 0b11111101;			// PZ1 == PA1
	PORTA &= (port >> 2) | 0b11111110;		// PZ2 == PA0
	PORTD &= (port >> 1) | 0b10000011;		// PZ3-PZ7 == PD2-PD6
	
}



// Перейти к следующему "шагу" в светодинамическом рисунке.
void next_step(void)
{
	if(bit_is_clear(mode,6)) {				// Движение по столбцам
		if( NEW_MODE ) {					// Если новый режим, то установить начальное значение
			PORTB = 0xFF;
			set_virt_port( start_pos[mode>>8] );	// mode >> 8 - индекс массива стартовых позиций
		}
		else
			set_virt_port( make_move(PORTZ) );
	}
	else if(bit_is_clear(mode,7)) {			// Движение по строкам
		if( NEW_MODE ) {					// Если новый режим, то установить начальное значение
			set_virt_port(0xFF);
			PORTB = start_pos[mode>>8];		// mode >> 8 - индекс массива стартовых позиций
		}
		else
			PORTB = make_move(PORTB);
	}
	else {									// Движение по строкам и столбцам
		if( NEW_MODE ) {					// Если новый режим, то установить начальное значение
			set_virt_port(0xFF);
			PORTB = start_pos[mode>>8];					// mode >> 8 - индекс массива стартовых позиций
			set_virt_port( start_pos[mode>>8] );
		}
		else {
			set_virt_port( make_move(PORTZ) );
			PORTB = make_move(PORTB);
		}
	}

	mode++;
	if(mode == 1280)		// Зацикливание рисунков (когда все режимы показаны - повторить снова)
		mode = 0;
}


// Определение режима: сдвиг или счет.
uchar make_move(uchar port) {
	if(bit_is_clear(mode, 5)) {				// если 5 бит в mode == 0, то режим сдвига
		if((mode & 0b00011111) == 31)		// вместо 32 шага сдвига - установка порта в 0 (что бы режим счета начинался с нуля)
			return 0;
		else
			return shift(port);
	}
	else
		return count(port);					// если 5 бит в mode == 1, то режим счета
}


// Режим счета
uchar count(uchar port) {
	if(bit_is_set(mode, 4))
		return count_right(port);		// если 4 бит в mode == 1, то режим счета вправо
	else
		return count_left(port);		// если 4 бит в mode == 0, то режим счета влево
}


// Счет вправо
uchar count_right(uchar port) {
	port >>= 1;							// сдвиг вправо
	if(bit_is_clear(mode, 3))			
		port |= 0b10000000;
	return port;
}


// Счет влево
uchar count_left(uchar port) {
	port <<= 1;							// сдвиг влево
	if(bit_is_clear(mode, 3))
		port |= 0b00000001;
	return port;
}


// Режим сдвига
uchar shift(uchar port) {
	if(bit_is_set(mode, 4))
		return shift_right(port);		// сдвиг вправо, если 4 бит в mode единица
	else
		return shift_left(port);		// иначе сдвиг влево
}


// Сдвиг влево
uchar shift_left(uchar port) {
	if(bit_is_set(port, 7)) {	// если самый левый бит единица, то после сдвига в единицу нужно установить самый правый бит
		port <<= 1;				// сдвиг влево на 1
		port |= 0b00000001;
	}
	else
		port <<= 1;
	return port;
}


// Сдвиг вправо
uchar shift_right(uchar port) {
	if(bit_is_set(port, 0)) {	// если самый правый бит единица, то после сдвига в единицу нужно установить самый левый бит
		port >>= 1;				// сдвиг вправо на 1
		port |= 0b10000000;	
	}
	else
		port >>= 1;
	return port;
}


